import * as cdk from 'aws-cdk-lib';
import * as s3 from 'aws-cdk-lib/aws-s3';
import * as cloudfront from 'aws-cdk-lib/aws-cloudfront';
import * as s3deploy from 'aws-cdk-lib/aws-s3-deployment';
import { S3StaticWebsiteOrigin } from 'aws-cdk-lib/aws-cloudfront-origins';
import { join } from 'path';

export class FrontendStack extends cdk.Stack {
  public readonly cloudfrontDistribution: cloudfront.Distribution;

  constructor(scope: cdk.App, id: string, props?: cdk.StackProps) {
    super(scope, id, props);

    // Create an S3 bucket for the frontend
    const frontendBucket = new s3.Bucket(this, 'ContactsAppFrontendBucket', {
      bucketName: 'contacts-app-frontend-bucket',
      websiteIndexDocument: 'index.html',
      websiteErrorDocument: 'index.html',
      removalPolicy: cdk.RemovalPolicy.DESTROY, // Only for dev environments
      publicReadAccess: false,
      blockPublicAccess: s3.BlockPublicAccess.BLOCK_ACLS,
    });

    // Set a bucket policy to allow public access
    frontendBucket.addToResourcePolicy(
      new cdk.aws_iam.PolicyStatement({
        actions: ['s3:GetObject'],
        resources: [`${frontendBucket.bucketArn}/*`], // Allow access to all objects in the bucket
        effect: cdk.aws_iam.Effect.ALLOW,
        principals: [new cdk.aws_iam.ArnPrincipal('*')], // Allow public access
      })
    );

    //find dir with frontend build
    const uiDir = join(__dirname, '..', '..', 'frontend', 'build');

    // Deploy the frontend files from the 'frontend' folder
    new s3deploy.BucketDeployment(this, 'DeployFrontend', {
      sources: [s3deploy.Source.asset(uiDir)], // Path to your frontend folder
      destinationBucket: frontendBucket,
    });

    // Create CloudFront distribution
    this.cloudfrontDistribution = new cloudfront.Distribution(
      this,
      'FrontendDistribution',
      {
        defaultBehavior: {
          origin: new S3StaticWebsiteOrigin(frontendBucket),
          viewerProtocolPolicy:
            cloudfront.ViewerProtocolPolicy.REDIRECT_TO_HTTPS, // Redirect HTTP to HTTPS
        },
      }
    );

    // Output the CloudFront URL
    new cdk.CfnOutput(this, 'CloudFrontClientAppURL', {
      value: `https://${this.cloudfrontDistribution.distributionDomainName}`,
      description: 'CloudFront URL for frontend',
    });
  }
}
